@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2019 MongoDB, Inc.
+ * Copyright (C) 2009-2020 MongoDB Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,6 +27,7 @@ static void pvt_replace_int32(byte_buffer_t *b, int32_t position, int32_t newval
 static void pvt_put_field(byte_buffer_t *b, VALUE rb_buffer, VALUE val, VALUE validating_keys);
 static void pvt_put_byte(byte_buffer_t *b, const char byte);
 static void pvt_put_int32(byte_buffer_t *b, const int32_t i32);
+static void pvt_put_uint32(byte_buffer_t *b, const uint32_t i32);
 static void pvt_put_int64(byte_buffer_t *b, const int64_t i);
 static void pvt_put_double(byte_buffer_t *b, double f);
 static void pvt_put_cstring(byte_buffer_t *b, const char *str, int32_t length, const char *data_type);
@@ -125,40 +126,51 @@ VALUE rb_bson_byte_buffer_put_bytes(VALUE self, VALUE bytes)
   return self;
 }
 
-/* write the byte denoting the BSON type for the passed object*/
+/* write the byte denoting the BSON type for the passed object */
 void pvt_put_type_byte(byte_buffer_t *b, VALUE val){
-  switch(TYPE(val)){
+  char type_byte;
+  
+  switch (TYPE(val)){
     case T_BIGNUM:
     case T_FIXNUM:
-      if(fits_int32(NUM2LL(val))){
-        pvt_put_byte(b, BSON_TYPE_INT32);
-      }else{
-        pvt_put_byte(b, BSON_TYPE_INT64);
+      if (fits_int32(NUM2LL(val))) {
+        type_byte = BSON_TYPE_INT32;
+      } else {
+        type_byte = BSON_TYPE_INT64;
       }
       break;
     case T_STRING:
-      pvt_put_byte(b, BSON_TYPE_STRING);
+      type_byte = BSON_TYPE_STRING;
       break;
     case T_ARRAY:
-      pvt_put_byte(b, BSON_TYPE_ARRAY);
+      type_byte = BSON_TYPE_ARRAY;
       break;
     case T_TRUE:
     case T_FALSE:
-      pvt_put_byte(b, BSON_TYPE_BOOLEAN);
+      type_byte = BSON_TYPE_BOOLEAN;
       break;
     case T_HASH:
-      pvt_put_byte(b, BSON_TYPE_DOCUMENT);
+      type_byte = BSON_TYPE_DOCUMENT;
       break;
     case T_FLOAT:
-      pvt_put_byte(b, BSON_TYPE_DOUBLE);
+      type_byte = BSON_TYPE_DOUBLE;
       break;
-    default:{
-      VALUE type = rb_funcall(val, rb_intern("bson_type"),0);
+    default: {
+      VALUE type;
+      VALUE responds = rb_funcall(val, rb_intern("respond_to?"), 1, ID2SYM(rb_intern("bson_type")));
+      if (!RTEST(responds)) {
+        VALUE klass = pvt_const_get_3("BSON", "Error", "UnserializableClass");
+        VALUE val_str = rb_funcall(val, rb_intern("to_s"), 0);
+        rb_raise(klass, "Value does not define its BSON serialized type: %s", RSTRING_PTR(val_str));
+      }
+      type = rb_funcall(val, rb_intern("bson_type"), 0);
+      type_byte = *RSTRING_PTR(type);
       RB_GC_GUARD(type);
-      pvt_put_byte(b, *RSTRING_PTR(type));
       break;
     }
   }
+  
+  pvt_put_byte(b, type_byte);
 }
 
 /**
@@ -379,6 +391,37 @@ void pvt_put_int32(byte_buffer_t *b, const int32_t i)
   b->write_position += 4;
 }
 
+/* The docstring is in init.c. */
+VALUE rb_bson_byte_buffer_put_uint32(VALUE self, VALUE i)
+{
+  byte_buffer_t *b;
+  int64_t temp;
+  uint32_t i32;
+
+  if (RB_TYPE_P(i, T_FLOAT)) {
+    rb_raise(rb_eArgError, "put_uint32: incorrect type: float, expected: integer");
+  }
+
+  temp = NUM2LL(i);
+  if (temp < 0 || temp > UINT32_MAX) {
+    rb_raise(rb_eRangeError, "Number %lld is out of range [0, 2^32)", (long long)temp);
+  }
+
+  i32 = NUM2UINT(i);
+
+  TypedData_Get_Struct(self, byte_buffer_t, &rb_byte_buffer_data_type, b);
+  pvt_put_uint32(b, i32);
+  return self;
+}
+
+void pvt_put_uint32(byte_buffer_t *b, const uint32_t i)
+{
+  const uint32_t i32 = BSON_UINT32_TO_LE(i);
+  ENSURE_BSON_WRITE(b, 4);
+  memcpy(WRITE_PTR(b), &i32, 4);
+  b->write_position += 4;
+}
+
 /* The docstring is in init.c. */
 VALUE rb_bson_byte_buffer_put_int64(VALUE self, VALUE i)
 {
@@ -623,7 +666,7 @@ VALUE rb_bson_byte_buffer_put_array(VALUE self, VALUE array, VALUE validating_ke
 
   for(int32_t index=0; index < RARRAY_LEN(array); index++, array_element++){
     pvt_put_type_byte(b, *array_element);
-    pvt_put_array_index(b,index);
+    pvt_put_array_index(b, index);
     pvt_put_field(b, self, *array_element, validating_keys);
   }
   pvt_put_byte(b, 0);
