@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2019 MongoDB Inc.
+# Copyright (C) 2009-2020 MongoDB Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,13 +31,22 @@ module BSON
     #
     # @param [ ByteBuffer ] buffer The byte buffer.
     #
+    # @option options [ nil | :bson ] :mode Decoding mode to use.
+    #
     # @return [ TrueClass, FalseClass ] The decoded boolean.
     #
     # @see http://bsonspec.org/#/specification
     #
     # @since 2.0.0
-    def self.from_bson(buffer)
-      buffer.get_byte == TrueClass::TRUE_BYTE
+    def self.from_bson(buffer, **options)
+      case v = buffer.get_byte
+      when TrueClass::TRUE_BYTE
+        true
+      when FalseClass::FALSE_BYTE
+        false
+      else
+        raise Error::BSONDecodeError, "Invalid boolean byte value: #{v}"
+      end
     end
 
     # Register this type when the module is loaded.
