@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2019 MongoDB Inc.
+# Copyright (C) 2009-2020 MongoDB Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,7 +35,7 @@ module BSON
     #
     # @since 2.1.0
     def to_bson(buffer = ByteBuffer.new, validating_keys = Config.validating_keys?)
-      to_time.to_bson(buffer)
+      gregorian.to_time.to_bson(buffer)
     end
   end
 
