@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2019 MongoDB, Inc.
+ * Copyright (C) 2015-2020 MongoDB Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ package org.bson;
 
 import java.io.ByteArrayOutputStream;
 import java.io.UnsupportedEncodingException;
+import java.math.BigInteger;
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
 import java.util.Arrays;
@@ -29,6 +30,7 @@ import org.jcodings.specific.UTF8Encoding;
 import org.jruby.Ruby;
 import org.jruby.RubyBignum;
 import org.jruby.RubyClass;
+import org.jruby.RubyException;
 import org.jruby.RubyFloat;
 import org.jruby.RubyFixnum;
 import org.jruby.RubyInteger;
@@ -36,8 +38,8 @@ import org.jruby.RubyNumeric;
 import org.jruby.RubyObject;
 import org.jruby.RubyString;
 import org.jruby.RubySymbol;
-import java.math.BigInteger;
 import org.jruby.anno.JRubyMethod;
+import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
@@ -96,6 +98,11 @@ public class ByteBuf extends RubyObject {
    */
   private int writePosition = 0;
 
+  /**
+   * The size of an unsigned 32-bit integer: 2^32 - 1
+   */
+  private static long UINT32_MAX = 4294967295L;
+  
   /**
    * Instantiate the ByteBuf - this is #allocate in Ruby.
    *
@@ -243,21 +250,21 @@ public class ByteBuf extends RubyObject {
   }
 
   /**
-   * Get a cstring from the buffer.
+   * Get a CString from the buffer.
    *
    * @author Durran Jordan
    * @since 2015.09.26
    * @version 4.0.0
    */
   @JRubyMethod(name = "get_cstring")
-  public RubyString getCString() {
+  public RubyString getCString(ThreadContext context) {
     ensureBsonRead();
     ByteArrayOutputStream bytes = new ByteArrayOutputStream();
     byte next = NULL_BYTE;
     while((next = this.buffer.get()) != NULL_BYTE) {
       bytes.write(next);
     }
-    RubyString string = getUTF8String(bytes.toByteArray());
+    RubyString string = getUTF8String(context, bytes.toByteArray());
     this.readPosition += (bytes.size() + 1);
     return string;
   }
@@ -304,6 +311,26 @@ public class ByteBuf extends RubyObject {
     return int32;
   }
 
+  /**
+   * Get a 32 bit integer from the buffer.
+   * 
+   */
+  @JRubyMethod(name = "get_uint32")
+  public RubyFixnum getUInt32() {
+    ensureBsonRead();
+
+    long temp = this.buffer.getInt();
+    // if temp is a negative number, convert to an unsigned 32 bit number
+    // by adding 2^32. For example if temp is -1, convert it to 2^32-1.
+    if (temp < 0) {
+      temp += UINT32_MAX + 1;
+    }
+    
+    RubyFixnum int32 = new RubyFixnum(getRuntime(), temp);
+    this.readPosition += 4;
+    return int32;
+  }
+
   /**
    * Get a UTF-8 string from the buffer.
    *
@@ -312,14 +339,24 @@ public class ByteBuf extends RubyObject {
    * @version 4.0.0
    */
   @JRubyMethod(name = "get_string")
-  public RubyString getString() {
+  public RubyString getString(ThreadContext context) {
     ensureBsonRead();
     int length = this.buffer.getInt();
     this.readPosition += 4;
     byte[] stringBytes = new byte[length];
     this.buffer.get(stringBytes);
+    if (stringBytes.length != length) {
+      RubyClass cls = context.runtime.getClass("BSON::Error::BSONDecodeError");
+      RubyString msg = RubyString.newString(context.runtime, String.format("Failed to read %d bytes: %d bytes read", length, stringBytes.length));
+      throw ((RubyException) cls.newInstance(context, msg, Block.NULL_BLOCK)).toThrowable();
+    }
+    if (stringBytes[stringBytes.length-1] != 0) {
+      RubyClass cls = context.runtime.getClass("BSON::Error::BSONDecodeError");
+      RubyString msg = RubyString.newString(context.runtime, "Last byte was not null: " + String.format("%02X", stringBytes[length-1]));
+      throw ((RubyException) cls.newInstance(context, msg, Block.NULL_BLOCK)).toThrowable();
+    }
     byte[] bytes = Arrays.copyOfRange(stringBytes, 0, stringBytes.length - 1);
-    RubyString string = getUTF8String(bytes);
+    RubyString string = getUTF8String(context, bytes);
     this.readPosition += length;
     return string;
   }
@@ -402,7 +439,7 @@ public class ByteBuf extends RubyObject {
   public ByteBuf putString(ThreadContext context, final IRubyObject value) throws UnsupportedEncodingException {
     RubyString string = (RubyString) value;
     RubyString encodedString = convertToUtf8(context, string);
-    
+
     String javaString = encodedString.asJavaString();
     return putJavaString(javaString);
   }
@@ -419,26 +456,26 @@ public class ByteBuf extends RubyObject {
   @JRubyMethod(name = "put_cstring")
   public ByteBuf putCString(ThreadContext context, final IRubyObject value) throws UnsupportedEncodingException {
 
-   if (value instanceof RubyFixnum) {
-     RubyString str = ((RubyFixnum) value).to_s();
-     String string = str.asJavaString();
-     this.writePosition += writeCharacters(string);
-   } else if (value instanceof RubyString || value instanceof RubySymbol) {
-    RubyString string;
-    if (value instanceof RubySymbol) {
-      string = (RubyString) ((RubySymbol) value).to_s(context);
+    if (value instanceof RubyFixnum) {
+      RubyString str = ((RubyFixnum) value).to_s();
+      String string = str.asJavaString();
+      this.writePosition += writeCharacters(string);
+    } else if (value instanceof RubyString || value instanceof RubySymbol) {
+      RubyString string;
+      if (value instanceof RubySymbol) {
+        string = (RubyString) ((RubySymbol) value).to_s(context);
+      } else {
+        string = (RubyString) value;
+      }
+      string = convertToUtf8(context, string);
+      String javaString = string.asJavaString();
+      verifyNoNulls(javaString);
+      this.writePosition += writeCharacters(javaString);
     } else {
-      string = (RubyString) value;
+      throw getRuntime().newTypeError(format("Invalid type for put_cstring: %s", value));
     }
-    string = convertToUtf8(context, string);
-    String javaString = string.asJavaString();
-    verifyNoNulls(javaString);
-    this.writePosition += writeCharacters(javaString);
-   } else {
-    throw getRuntime().newTypeError(format("Invalid type for put_cstring: %s", value));
-   }
 
-   return this;
+    return this;
   }
 
   /**
@@ -476,6 +513,33 @@ public class ByteBuf extends RubyObject {
     return this;
   }
 
+  /**
+   * Put an unsigned 32 bit integer onto the buffer.
+   *
+   * @param value The integer to write.
+   * 
+   */
+  @JRubyMethod(name = "put_uint32")
+  public ByteBuf putUInt32(ThreadContext context, IRubyObject value) {
+    if (value instanceof RubyFloat) {
+      throw getRuntime().newArgumentError("put_uint32: incorrect type: float, expected: integer");
+    }
+    ensureBsonWrite(4);
+
+    long temp = RubyNumeric.fix2long((RubyFixnum) value);
+    
+    if (temp > UINT32_MAX || temp < 0) {
+      throw getRuntime().newRangeError(format("Number %d is out of range [0, 2^32)", temp));
+    }
+
+    // When a long is cast to an int, Java appears to take the bits of the long and 
+    // use them as is for the int value. For example, if temp is 2^32-1, (int) temp
+    // would be -1, and if temp is 2^31, (int) temp would be -2^31.
+    this.buffer.putInt((int) temp);
+    this.writePosition += 4;
+    return this;
+  }
+
   /**
    * Put a 64 bit integer onto the buffer.
    *
@@ -658,10 +722,15 @@ public class ByteBuf extends RubyObject {
     }
   }
 
-  private RubyString getUTF8String(final byte[] bytes) {
-    return RubyString.newString(getRuntime(), new ByteList(bytes, UTF_8));
+  private RubyString getUTF8String(ThreadContext context, final byte[] bytes) {
+    // This call appears to not validate that the byte sequence is valid UTF-8
+    RubyString str = RubyString.newString(context.runtime, bytes, 0, bytes.length, UTF_8);
+    // ... hence validate manually:
+    convertToUtf8(context, str);
+    
+    return str;
   }
-  
+
   /**
    * Converts +string+ to UTF-8 encoding. If +string+ is already in UTF-8,
    * verifies that +string+ contains valid byte sequences by encoding in
@@ -669,7 +738,7 @@ public class ByteBuf extends RubyObject {
    */
   private RubyString convertToUtf8(ThreadContext context, RubyString string) {
     RubyString encodedString;
-    
+
     if (string.getEncoding() == UTF8Encoding.INSTANCE) {
       // If the value is already in UTF-8, encoding it to UTF-8 is a noop.
       // But we also want to validate the bytes for being a valid UTF-8 sequence.
@@ -681,10 +750,10 @@ public class ByteBuf extends RubyObject {
       RubyString utf8 = RubyString.newString(context.runtime, "UTF-8");
       encodedString = (RubyString) string.encode(context, utf8);
     }
-    
+
     return encodedString;
   }
-  
+
   private void verifyNoNulls(String string) {
     int len = string.length();
 
